@@ -4,6 +4,8 @@ Tests for `attr._make`.
 
 from __future__ import absolute_import, division, print_function
 
+import copy
+import gc
 import inspect
 import itertools
 import sys
@@ -18,17 +20,36 @@ from hypothesis.strategies import booleans, integers, lists, sampled_from, text
 import attr
 
 from attr import _config
-from attr._compat import PY2
+from attr._compat import PY2, ordered_dict
 from attr._make import (
-    Attribute, Factory, _AndValidator, _Attributes, _ClassBuilder,
-    _CountingAttr, _transform_attrs, and_, fields, make_class, validate
+    Attribute,
+    Factory,
+    _AndValidator,
+    _Attributes,
+    _ClassBuilder,
+    _CountingAttr,
+    _transform_attrs,
+    and_,
+    fields,
+    fields_dict,
+    make_class,
+    validate,
+)
+from attr.exceptions import (
+    DefaultAlreadySetError,
+    NotAnAttrsClassError,
+    PythonTooOldError,
 )
-from attr.exceptions import DefaultAlreadySetError, NotAnAttrsClassError
 
-from .utils import (
-    gen_attr_names, list_of_attrs, simple_attr, simple_attrs,
-    simple_attrs_with_metadata, simple_attrs_without_metadata, simple_classes
+from .strategies import (
+    gen_attr_names,
+    list_of_attrs,
+    simple_attrs,
+    simple_attrs_with_metadata,
+    simple_attrs_without_metadata,
+    simple_classes,
 )
+from .utils import simple_attr
 
 
 attrs_st = simple_attrs.map(lambda c: Attribute.from_counting_attr("name", c))
@@ -38,6 +59,7 @@ class TestCountingAttr(object):
     """
     Tests for `attr`.
     """
+
     def test_returns_Attr(self):
         """
         Returns an instance of _CountingAttr.
@@ -50,6 +72,7 @@ class TestCountingAttr(object):
         """
         If a list is passed as validator, it's just converted to a tuple.
         """
+
         def v1(_, __):
             pass
 
@@ -58,7 +81,7 @@ class TestCountingAttr(object):
 
         a = attr.ib(validator=[v1, v2])
 
-        assert _AndValidator((v1, v2,)) == a._validator
+        assert _AndValidator((v1, v2)) == a._validator
 
     def test_validator_decorator_single(self):
         """
@@ -73,17 +96,15 @@ class TestCountingAttr(object):
 
         assert v == a._validator
 
-    @pytest.mark.parametrize("wrap", [
-        lambda v: v,
-        lambda v: [v],
-        lambda v: and_(v)
-
-    ])
+    @pytest.mark.parametrize(
+        "wrap", [lambda v: v, lambda v: [v], lambda v: and_(v)]
+    )
     def test_validator_decorator(self, wrap):
         """
         If _CountingAttr.validator is used as a decorator and there is already
         a decorator set, the decorators are composed using `and_`.
         """
+
         def v(_, __):
             pass
 
@@ -93,7 +114,7 @@ class TestCountingAttr(object):
         def v2(self, _, __):
             pass
 
-        assert _AndValidator((v, v2,)) == a._validator
+        assert _AndValidator((v, v2)) == a._validator
 
     def test_default_decorator_already_set(self):
         """
@@ -103,6 +124,7 @@ class TestCountingAttr(object):
         a = attr.ib(default=42)
 
         with pytest.raises(DefaultAlreadySetError):
+
             @a.default
             def f(self):
                 pass
@@ -125,11 +147,13 @@ class TestAttribute(object):
     """
     Tests for `attr.Attribute`.
     """
+
     def test_deprecated_convert_argument(self):
         """
         Using *convert* raises a DeprecationWarning and sets the converter
         field.
         """
+
         def conv(v):
             return v
 
@@ -150,6 +174,7 @@ class TestAttribute(object):
         """
         If Attribute.convert is accessed, a DeprecationWarning is raised.
         """
+
         def conv(v):
             return v
 
@@ -171,8 +196,15 @@ class TestAttribute(object):
         """
         with pytest.raises(RuntimeError) as ei:
             Attribute(
-                "a", True, True, True, True, True, True,
-                convert=lambda v: v, converter=lambda v: v,
+                "a",
+                True,
+                True,
+                True,
+                True,
+                True,
+                True,
+                convert=lambda v: v,
+                converter=lambda v: v,
             )
 
         assert (
@@ -187,6 +219,7 @@ def make_tc():
         y = attr.ib()
         x = attr.ib()
         a = 42
+
     return TransformC
 
 
@@ -194,12 +227,13 @@ class TestTransformAttrs(object):
     """
     Tests for `_transform_attrs`.
     """
+
     def test_no_modifications(self):
         """
-        Doesn't attach __attrs_attrs__ to the class anymore.
+        Does not attach __attrs_attrs__ to the class.
         """
         C = make_tc()
-        _transform_attrs(C, None, False)
+        _transform_attrs(C, None, False, False)
 
         assert None is getattr(C, "__attrs_attrs__", None)
 
@@ -208,7 +242,7 @@ class TestTransformAttrs(object):
         Transforms every `_CountingAttr` and leaves others (a) be.
         """
         C = make_tc()
-        attrs, _, = _transform_attrs(C, None, False)
+        attrs, _, _ = _transform_attrs(C, None, False, False)
 
         assert ["z", "y", "x"] == [a.name for a in attrs]
 
@@ -216,20 +250,23 @@ class TestTransformAttrs(object):
         """
         No attributes works as expected.
         """
+
         @attr.s
         class C(object):
             pass
 
-        assert _Attributes(((), [])) == _transform_attrs(C, None, False)
+        assert _Attributes(((), [], {})) == _transform_attrs(
+            C, None, False, False
+        )
 
     def test_transforms_to_attribute(self):
         """
         All `_CountingAttr`s are transformed into `Attribute`s.
         """
         C = make_tc()
-        attrs, super_attrs = _transform_attrs(C, None, False)
+        attrs, base_attrs, _ = _transform_attrs(C, None, False, False)
 
-        assert [] == super_attrs
+        assert [] == base_attrs
         assert 3 == len(attrs)
         assert all(isinstance(a, Attribute) for a in attrs)
 
@@ -238,36 +275,95 @@ class TestTransformAttrs(object):
         Raises `ValueError` if attributes with defaults are followed by
         mandatory attributes.
         """
+
         class C(object):
             x = attr.ib(default=None)
             y = attr.ib()
 
         with pytest.raises(ValueError) as e:
-            _transform_attrs(C, None, False)
+            _transform_attrs(C, None, False, False)
         assert (
             "No mandatory attributes allowed after an attribute with a "
             "default value or factory.  Attribute in question: Attribute"
             "(name='y', default=NOTHING, validator=None, repr=True, "
             "cmp=True, hash=None, init=True, metadata=mappingproxy({}), "
-            "type=None, converter=None)",
+            "type=None, converter=None, kw_only=False)",
         ) == e.value.args
 
+    def test_kw_only(self):
+        """
+        Converts all attributes, including base class' attributes, if `kw_only`
+        is provided. Therefore, `kw_only` allows attributes with defaults to
+        preceed mandatory attributes.
+
+        Updates in the subclass *don't* affect the base class attributes.
+        """
+
+        @attr.s
+        class B(object):
+            b = attr.ib()
+
+        for b_a in B.__attrs_attrs__:
+            assert b_a.kw_only is False
+
+        class C(B):
+            x = attr.ib(default=None)
+            y = attr.ib()
+
+        attrs, base_attrs, _ = _transform_attrs(C, None, False, True)
+
+        assert len(attrs) == 3
+        assert len(base_attrs) == 1
+
+        for a in attrs:
+            assert a.kw_only is True
+
+        for b_a in B.__attrs_attrs__:
+            assert b_a.kw_only is False
+
     def test_these(self):
         """
-        If these is passed, use it and ignore body and super classes.
+        If these is passed, use it and ignore body and base classes.
         """
+
         class Base(object):
             z = attr.ib()
 
         class C(Base):
             y = attr.ib()
 
-        attrs, super_attrs = _transform_attrs(C, {"x": attr.ib()}, False)
+        attrs, base_attrs, _ = _transform_attrs(
+            C, {"x": attr.ib()}, False, False
+        )
 
-        assert [] == super_attrs
-        assert (
-            simple_attr("x"),
-        ) == attrs
+        assert [] == base_attrs
+        assert (simple_attr("x"),) == attrs
+
+    def test_these_leave_body(self):
+        """
+        If these is passed, no attributes are removed from the body.
+        """
+
+        @attr.s(init=False, these={"x": attr.ib()})
+        class C(object):
+            x = 5
+
+        assert 5 == C().x
+        assert "C(x=5)" == repr(C())
+
+    def test_these_ordered(self):
+        """
+        If these is passed ordered attrs, their order respect instead of the
+        counter.
+        """
+        b = attr.ib(default=2)
+        a = attr.ib(default=1)
+
+        @attr.s(these=ordered_dict([("a", a), ("b", b)]))
+        class C(object):
+            pass
+
+        assert "C(a=1, b=2)" == repr(C())
 
     def test_multiple_inheritance(self):
         """
@@ -275,6 +371,7 @@ class TestTransformAttrs(object):
 
         See #285
         """
+
         @attr.s
         class A(object):
             a1 = attr.ib(default="a1")
@@ -310,12 +407,14 @@ class TestAttributes(object):
     """
     Tests for the `attrs`/`attr.s` class decorator.
     """
+
     @pytest.mark.skipif(not PY2, reason="No old-style classes in Py3")
     def test_catches_old_style(self):
         """
         Raises TypeError on old-style classes.
         """
         with pytest.raises(TypeError) as e:
+
             @attr.s
             class C:
                 pass
@@ -326,6 +425,7 @@ class TestAttributes(object):
         """
         Sets the `__attrs_attrs__` class attribute with a list of `Attribute`s.
         """
+
         @attr.s
         class C(object):
             x = attr.ib()
@@ -337,6 +437,7 @@ class TestAttributes(object):
         """
         No attributes, no problems.
         """
+
         @attr.s
         class C3(object):
             pass
@@ -352,12 +453,9 @@ class TestAttributes(object):
         with pytest.raises(AttributeError):
             setattr(attr, attr_name, 1)
 
-    @pytest.mark.parametrize("method_name", [
-        "__repr__",
-        "__eq__",
-        "__hash__",
-        "__init__",
-    ])
+    @pytest.mark.parametrize(
+        "method_name", ["__repr__", "__eq__", "__hash__", "__init__"]
+    )
     def test_adds_all_by_default(self, method_name):
         """
         If no further arguments are supplied, all add_XXX functions except
@@ -379,12 +477,15 @@ class TestAttributes(object):
         if method_name == "__hash__":
             assert meth is None
 
-    @pytest.mark.parametrize("arg_name, method_name", [
-        ("repr", "__repr__"),
-        ("cmp", "__eq__"),
-        ("hash", "__hash__"),
-        ("init", "__init__"),
-    ])
+    @pytest.mark.parametrize(
+        "arg_name, method_name",
+        [
+            ("repr", "__repr__"),
+            ("cmp", "__eq__"),
+            ("hash", "__hash__"),
+            ("init", "__init__"),
+        ],
+    )
     def test_respects_add_arguments(self, arg_name, method_name):
         """
         If a certain `add_XXX` is `False`, `__XXX__` is not added to the class.
@@ -393,12 +494,7 @@ class TestAttributes(object):
         # overwritten afterwards.
         sentinel = object()
 
-        am_args = {
-            "repr": True,
-            "cmp": True,
-            "hash": True,
-            "init": True
-        }
+        am_args = {"repr": True, "cmp": True, "hash": True, "init": True}
         am_args[arg_name] = False
 
         class C(object):
@@ -416,6 +512,7 @@ class TestAttributes(object):
         """
         On Python 3, the name in repr is the __qualname__.
         """
+
         @attr.s(slots=slots_outer)
         class C(object):
             @attr.s(slots=slots_inner)
@@ -430,11 +527,13 @@ class TestAttributes(object):
         """
         Setting repr_ns overrides a potentially guessed namespace.
         """
+
         @attr.s(slots=slots_outer)
         class C(object):
             @attr.s(repr_ns="C", slots=slots_inner)
             class D(object):
                 pass
+
         assert "C.D()" == repr(C.D())
 
     @pytest.mark.skipif(PY2, reason="__qualname__ is PY3-only.")
@@ -443,6 +542,7 @@ class TestAttributes(object):
         """
         On Python 3, __name__ is different from __qualname__.
         """
+
         @attr.s(slots=slots_outer)
         class C(object):
             @attr.s(slots=slots_inner)
@@ -469,12 +569,13 @@ class TestAttributes(object):
 
         c = C(x=10, y=20)
 
-        assert 30 == getattr(c, 'z', None)
+        assert 30 == getattr(c, "z", None)
 
     def test_types(self):
         """
         Sets the `Attribute.type` attr from type argument.
         """
+
         @attr.s
         class C(object):
             x = attr.ib(type=int)
@@ -490,6 +591,7 @@ class TestAttributes(object):
         """
         Attribute definitions do not appear on the class body after @attr.s.
         """
+
         @attr.s(slots=slots)
         class C(object):
             x = attr.ib()
@@ -498,6 +600,214 @@ class TestAttributes(object):
 
         assert not isinstance(x, _CountingAttr)
 
+    def test_factory_sugar(self):
+        """
+        Passing factory=f is syntactic sugar for passing default=Factory(f).
+        """
+
+        @attr.s
+        class C(object):
+            x = attr.ib(factory=list)
+
+        assert Factory(list) == attr.fields(C).x.default
+
+    def test_sugar_factory_mutex(self):
+        """
+        Passing both default and factory raises ValueError.
+        """
+        with pytest.raises(ValueError, match="mutually exclusive"):
+
+            @attr.s
+            class C(object):
+                x = attr.ib(factory=list, default=Factory(list))
+
+    def test_sugar_callable(self):
+        """
+        Factory has to be a callable to prevent people from passing Factory
+        into it.
+        """
+        with pytest.raises(ValueError, match="must be a callable"):
+
+            @attr.s
+            class C(object):
+                x = attr.ib(factory=Factory(list))
+
+
+@pytest.mark.skipif(PY2, reason="keyword-only arguments are PY3-only.")
+class TestKeywordOnlyAttributes(object):
+    """
+    Tests for keyword-only attributes.
+    """
+
+    def test_adds_keyword_only_arguments(self):
+        """
+        Attributes can be added as keyword-only.
+        """
+
+        @attr.s
+        class C(object):
+            a = attr.ib()
+            b = attr.ib(default=2, kw_only=True)
+            c = attr.ib(kw_only=True)
+            d = attr.ib(default=attr.Factory(lambda: 4), kw_only=True)
+
+        c = C(1, c=3)
+
+        assert c.a == 1
+        assert c.b == 2
+        assert c.c == 3
+        assert c.d == 4
+
+    def test_ignores_kw_only_when_init_is_false(self):
+        """
+        Specifying ``kw_only=True`` when ``init=False`` is essentially a no-op.
+        """
+
+        @attr.s
+        class C(object):
+            x = attr.ib(init=False, default=0, kw_only=True)
+            y = attr.ib()
+
+        c = C(1)
+
+        assert c.x == 0
+        assert c.y == 1
+
+    def test_keyword_only_attributes_presence(self):
+        """
+        Raises `TypeError` when keyword-only arguments are
+        not specified.
+        """
+
+        @attr.s
+        class C(object):
+            x = attr.ib(kw_only=True)
+
+        with pytest.raises(TypeError) as e:
+            C()
+
+        assert (
+            "missing 1 required keyword-only argument: 'x'"
+        ) in e.value.args[0]
+
+    def test_conflicting_keyword_only_attributes(self):
+        """
+        Raises `ValueError` if keyword-only attributes are followed by
+        regular (non keyword-only) attributes.
+        """
+
+        class C(object):
+            x = attr.ib(kw_only=True)
+            y = attr.ib()
+
+        with pytest.raises(ValueError) as e:
+            _transform_attrs(C, None, False, False)
+
+        assert (
+            "Non keyword-only attributes are not allowed after a "
+            "keyword-only attribute.  Attribute in question: Attribute"
+            "(name='y', default=NOTHING, validator=None, repr=True, "
+            "cmp=True, hash=None, init=True, metadata=mappingproxy({}), "
+            "type=None, converter=None, kw_only=False)",
+        ) == e.value.args
+
+    def test_keyword_only_attributes_allow_subclassing(self):
+        """
+        Subclass can define keyword-only attributed without defaults,
+        when the base class has attributes with defaults.
+        """
+
+        @attr.s
+        class Base(object):
+            x = attr.ib(default=0)
+
+        @attr.s
+        class C(Base):
+            y = attr.ib(kw_only=True)
+
+        c = C(y=1)
+
+        assert c.x == 0
+        assert c.y == 1
+
+    def test_keyword_only_class_level(self):
+        """
+        `kw_only` can be provided at the attr.s level, converting all
+        attributes to `kw_only.`
+        """
+
+        @attr.s(kw_only=True)
+        class C:
+            x = attr.ib()
+            y = attr.ib(kw_only=True)
+
+        with pytest.raises(TypeError):
+            C(0, y=1)
+
+        c = C(x=0, y=1)
+
+        assert c.x == 0
+        assert c.y == 1
+
+    def test_keyword_only_class_level_subclassing(self):
+        """
+        Subclass `kw_only` propagates to attrs inherited from the base,
+        allowing non-default following default.
+        """
+
+        @attr.s
+        class Base(object):
+            x = attr.ib(default=0)
+
+        @attr.s(kw_only=True)
+        class C(Base):
+            y = attr.ib()
+
+        with pytest.raises(TypeError):
+            C(1)
+
+        c = C(x=0, y=1)
+
+        assert c.x == 0
+        assert c.y == 1
+
+
+@pytest.mark.skipif(not PY2, reason="PY2-specific keyword-only error behavior")
+class TestKeywordOnlyAttributesOnPy2(object):
+    """
+    Tests for keyword-only attribute behavior on py2.
+    """
+
+    def test_syntax_error(self):
+        """
+        Keyword-only attributes raise Syntax error on ``__init__`` generation.
+        """
+
+        with pytest.raises(PythonTooOldError):
+
+            @attr.s(kw_only=True)
+            class ClassLevel(object):
+                a = attr.ib()
+
+        with pytest.raises(PythonTooOldError):
+
+            @attr.s()
+            class AttrLevel(object):
+                a = attr.ib(kw_only=True)
+
+    def test_no_init(self):
+        """
+        Keyworld-only is a no-op, not any error, if ``init=false``.
+        """
+
+        @attr.s(kw_only=True, init=False)
+        class ClassLevel(object):
+            a = attr.ib()
+
+        @attr.s(init=False)
+        class AttrLevel(object):
+            a = attr.ib(kw_only=True)
+
 
 @attr.s
 class GC(object):
@@ -510,10 +820,8 @@ class TestMakeClass(object):
     """
     Tests for `make_class`.
     """
-    @pytest.mark.parametrize("ls", [
-        list,
-        tuple
-    ])
+
+    @pytest.mark.parametrize("ls", [list, tuple])
     def test_simple(self, ls):
         """
         Passing a list of strings creates attributes with default args.
@@ -531,10 +839,9 @@ class TestMakeClass(object):
         """
         Passing a dict of name: _CountingAttr creates an equivalent class.
         """
-        C1 = make_class("C1", {
-            "a": attr.ib(default=42),
-            "b": attr.ib(default=None),
-        })
+        C1 = make_class(
+            "C1", {"a": attr.ib(default=42), "b": attr.ib(default=None)}
+        )
 
         @attr.s
         class C2(object):
@@ -558,14 +865,13 @@ class TestMakeClass(object):
         with pytest.raises(TypeError) as e:
             make_class("C", object())
 
-        assert (
-            "attrs argument must be a dict or a list.",
-        ) == e.value.args
+        assert ("attrs argument must be a dict or a list.",) == e.value.args
 
     def test_bases(self):
         """
         Parameter bases default to (object,) and subclasses correctly
         """
+
         class D(object):
             pass
 
@@ -593,16 +899,29 @@ class TestMakeClass(object):
         """
         `make_class()` does not fail when `sys._getframe()` is not available.
         """
-        monkeypatch.delattr(sys, '_getframe')
+        monkeypatch.delattr(sys, "_getframe")
         C = make_class("C", ["x"])
 
         assert 1 == len(C.__attrs_attrs__)
 
+    def test_make_class_ordered(self):
+        """
+        If `make_class()` is passed ordered attrs, their order is respected
+        instead of the counter.
+        """
+        b = attr.ib(default=2)
+        a = attr.ib(default=1)
+
+        C = attr.make_class("C", ordered_dict([("a", a), ("b", b)]))
+
+        assert "C(a=1, b=2)" == repr(C())
+
 
 class TestFields(object):
     """
     Tests for `fields`.
     """
+
     def test_instance(self, C):
         """
         Raises `TypeError` on non-classes.
@@ -618,6 +937,7 @@ class TestFields(object):
         """
         with pytest.raises(NotAnAttrsClassError) as e:
             fields(object)
+
         assert (
             "{o!r} is not an attrs-decorated class.".format(o=object)
         ) == e.value.args[0]
@@ -638,18 +958,55 @@ class TestFields(object):
             assert getattr(fields(C), attribute.name) is attribute
 
 
+class TestFieldsDict(object):
+    """
+    Tests for `fields_dict`.
+    """
+
+    def test_instance(self, C):
+        """
+        Raises `TypeError` on non-classes.
+        """
+        with pytest.raises(TypeError) as e:
+            fields_dict(C(1, 2))
+
+        assert "Passed object must be a class." == e.value.args[0]
+
+    def test_handler_non_attrs_class(self, C):
+        """
+        Raises `ValueError` if passed a non-``attrs`` instance.
+        """
+        with pytest.raises(NotAnAttrsClassError) as e:
+            fields_dict(object)
+
+        assert (
+            "{o!r} is not an attrs-decorated class.".format(o=object)
+        ) == e.value.args[0]
+
+    @given(simple_classes())
+    def test_fields_dict(self, C):
+        """
+        Returns an ordered dict of ``{attribute_name: Attribute}``.
+        """
+        d = fields_dict(C)
+
+        assert isinstance(d, ordered_dict)
+        assert list(fields(C)) == list(d.values())
+        assert [a.name for a in fields(C)] == [field_name for field_name in d]
+
+
 class TestConverter(object):
     """
     Tests for attribute conversion.
     """
+
     def test_convert(self):
         """
         Return value of converter is used as the attribute's value.
         """
-        C = make_class("C", {
-            "x": attr.ib(converter=lambda v: v + 1),
-            "y": attr.ib(),
-        })
+        C = make_class(
+            "C", {"x": attr.ib(converter=lambda v: v + 1), "y": attr.ib()}
+        )
         c = C(1, 2)
 
         assert c.x == 2
@@ -660,10 +1017,15 @@ class TestConverter(object):
         """
         Property tests for attributes with convert.
         """
-        C = make_class("C", {
-            "y": attr.ib(),
-            "x": attr.ib(init=init, default=val, converter=lambda v: v + 1),
-        })
+        C = make_class(
+            "C",
+            {
+                "y": attr.ib(),
+                "x": attr.ib(
+                    init=init, default=val, converter=lambda v: v + 1
+                ),
+            },
+        )
         c = C(2)
 
         assert c.x == val + 1
@@ -674,13 +1036,22 @@ class TestConverter(object):
         """
         Property tests for attributes with convert, and a factory default.
         """
-        C = make_class("C", {
-            "y": attr.ib(),
-            "x": attr.ib(
-                init=init,
-                default=Factory(lambda: val),
-                converter=lambda v: v + 1),
-        })
+        C = make_class(
+            "C",
+            ordered_dict(
+                [
+                    ("y", attr.ib()),
+                    (
+                        "x",
+                        attr.ib(
+                            init=init,
+                            default=Factory(lambda: val),
+                            converter=lambda v: v + 1,
+                        ),
+                    ),
+                ]
+            ),
+        )
         c = C(2)
 
         assert c.x == val + 1
@@ -690,11 +1061,14 @@ class TestConverter(object):
         """
         If takes_self on factories is True, self is passed.
         """
-        C = make_class("C", {
-            "x": attr.ib(
-                default=Factory((lambda self: self), takes_self=True)
-            ),
-        })
+        C = make_class(
+            "C",
+            {
+                "x": attr.ib(
+                    default=Factory((lambda self: self), takes_self=True)
+                )
+            },
+        )
 
         i = C()
 
@@ -710,13 +1084,17 @@ class TestConverter(object):
         """
         Validation happens after conversion.
         """
+
         def validator(inst, attr, val):
             raise RuntimeError("foo")
+
         C = make_class(
-            "C", {
+            "C",
+            {
                 "x": attr.ib(validator=validator, converter=lambda v: 1 / 0),
                 "y": attr.ib(),
-            })
+            },
+        )
         with pytest.raises(ZeroDivisionError):
             C(1, 2)
 
@@ -724,9 +1102,9 @@ class TestConverter(object):
         """
         Converters circumvent immutability.
         """
-        C = make_class("C", {
-            "x": attr.ib(converter=lambda v: int(v)),
-        }, frozen=True)
+        C = make_class(
+            "C", {"x": attr.ib(converter=lambda v: int(v))}, frozen=True
+        )
         C("1")
 
     def test_deprecated_convert(self):
@@ -734,10 +1112,12 @@ class TestConverter(object):
         Using *convert* raises a DeprecationWarning and sets the converter
         field.
         """
+
         def conv(v):
             return v
 
         with pytest.warns(DeprecationWarning) as wi:
+
             @attr.s
             class C(object):
                 x = attr.ib(convert=conv)
@@ -759,6 +1139,7 @@ class TestConverter(object):
         A TypeError is raised if both *convert* and *converter* are passed.
         """
         with pytest.raises(RuntimeError) as ei:
+
             @attr.s
             class C(object):
                 x = attr.ib(convert=lambda v: v, converter=lambda v: v)
@@ -773,20 +1154,21 @@ class TestValidate(object):
     """
     Tests for `validate`.
     """
+
     def test_success(self):
         """
         If the validator succeeds, nothing gets raised.
         """
-        C = make_class("C", {
-            "x": attr.ib(validator=lambda *a: None),
-            "y": attr.ib()
-        })
+        C = make_class(
+            "C", {"x": attr.ib(validator=lambda *a: None), "y": attr.ib()}
+        )
         validate(C(1, 2))
 
     def test_propagates(self):
         """
         The exception of the validator is handed through.
         """
+
         def raiser(_, __, value):
             if value == 42:
                 raise FloatingPointError
@@ -826,6 +1208,7 @@ class TestValidate(object):
         If a list is passed as a validator, all of its items are treated as one
         and must pass.
         """
+
         def v1(_, __, value):
             if value == 23:
                 raise TypeError("omg")
@@ -861,13 +1244,15 @@ class TestValidate(object):
 # Hypothesis seems to cache values, so the lists of attributes come out
 # unsorted.
 sorted_lists_of_attrs = list_of_attrs.map(
-    lambda l: sorted(l, key=attrgetter("counter")))
+    lambda l: sorted(l, key=attrgetter("counter"))
+)
 
 
 class TestMetadata(object):
     """
     Tests for metadata handling.
     """
+
     @given(sorted_lists_of_attrs)
     def test_metadata_present(self, list_of_attrs):
         """
@@ -886,8 +1271,9 @@ class TestMetadata(object):
                 # Once more, just to assert getting items and iteration.
                 for k in class_attr.metadata:
                     assert hyp_attr.metadata[k] == class_attr.metadata[k]
-                    assert (hyp_attr.metadata.get(k) ==
-                            class_attr.metadata.get(k))
+                    assert hyp_attr.metadata.get(k) == class_attr.metadata.get(
+                        k
+                    )
 
     @given(simple_classes(), text())
     def test_metadata_immutability(self, C, string):
@@ -912,7 +1298,7 @@ class TestMetadata(object):
                 with pytest.raises(AttributeError):
                     a.metadata.pop(k)
             with pytest.raises(AttributeError):
-                    a.metadata.popitem()
+                a.metadata.popitem()
 
     @given(lists(simple_attrs_without_metadata, min_size=2, max_size=5))
     def test_empty_metadata_singleton(self, list_of_attrs):
@@ -961,6 +1347,7 @@ class TestClassBuilder(object):
     """
     Tests for `_ClassBuilder`.
     """
+
     def test_repr_str(self):
         """
         Trying to add a `__str__` without having a `__repr__` raises a
@@ -977,10 +1364,11 @@ class TestClassBuilder(object):
         """
         repr of builder itself makes sense.
         """
+
         class C(object):
             pass
 
-        b = _ClassBuilder(C, None, True, True, False)
+        b = _ClassBuilder(C, None, True, True, False, False, False, False)
 
         assert "<_ClassBuilder(cls=C)>" == repr(b)
 
@@ -988,25 +1376,44 @@ class TestClassBuilder(object):
         """
         All methods return the builder for chaining.
         """
+
         class C(object):
             x = attr.ib()
 
-        b = _ClassBuilder(C, None, True, True, False)
+        b = _ClassBuilder(C, None, True, True, False, False, False, False)
 
-        cls = b.add_cmp().add_hash().add_init().add_repr("ns").add_str() \
+        cls = (
+            b.add_cmp()
+            .add_hash()
+            .add_init()
+            .add_repr("ns")
+            .add_str()
             .build_class()
+        )
 
         assert "ns.C(x=1)" == repr(cls(1))
 
-    @pytest.mark.parametrize("meth_name", [
-        "__init__", "__hash__", "__repr__", "__str__",
-        "__eq__", "__ne__", "__lt__", "__le__", "__gt__", "__ge__",
-    ])
+    @pytest.mark.parametrize(
+        "meth_name",
+        [
+            "__init__",
+            "__hash__",
+            "__repr__",
+            "__str__",
+            "__eq__",
+            "__ne__",
+            "__lt__",
+            "__le__",
+            "__gt__",
+            "__ge__",
+        ],
+    )
     def test_attaches_meta_dunders(self, meth_name):
         """
         Generated methods have correct __module__, __name__, and __qualname__
         attributes.
         """
+
         @attr.s(hash=True, str=True)
         class C(object):
             def organic(self):
@@ -1025,11 +1432,19 @@ class TestClassBuilder(object):
         If the class hasn't a __module__ or __qualname__, the method hasn't
         either.
         """
+
         class C(object):
             pass
 
         b = _ClassBuilder(
-            C, these=None, slots=False, frozen=False, auto_attribs=False,
+            C,
+            these=None,
+            slots=False,
+            frozen=False,
+            weakref_slot=True,
+            auto_attribs=False,
+            kw_only=False,
+            cache_hash=False,
         )
         b._cls = {}  # no __module__; no __qualname__
 
@@ -1043,3 +1458,75 @@ class TestClassBuilder(object):
 
         assert "42" == rv.__module__ == fake_meth.__module__
         assert "23" == rv.__qualname__ == fake_meth.__qualname__
+
+    def test_weakref_setstate(self):
+        """
+        __weakref__ is not set on in setstate because it's not writable in
+        slots classes.
+        """
+
+        @attr.s(slots=True)
+        class C(object):
+            __weakref__ = attr.ib(
+                init=False, hash=False, repr=False, cmp=False
+            )
+
+        assert C() == copy.deepcopy(C())
+
+    def test_no_references_to_original(self):
+        """
+        When subclassing a slots class, there are no stray references to the
+        original class.
+        """
+
+        @attr.s(slots=True)
+        class C(object):
+            pass
+
+        @attr.s(slots=True)
+        class C2(C):
+            pass
+
+        # The original C2 is in a reference cycle, so force a collect:
+        gc.collect()
+
+        assert [C2] == C.__subclasses__()
+
+
+class TestMakeCmp:
+    """
+    Tests for _make_cmp().
+    """
+
+    @pytest.mark.parametrize(
+        "op", ["__%s__" % (op,) for op in ("lt", "le", "gt", "ge")]
+    )
+    def test_subclasses_deprecated(self, recwarn, op):
+        """
+        Calling comparison methods on subclasses raises a deprecation warning;
+        calling them on identical classes does not..
+        """
+
+        @attr.s
+        class A(object):
+            a = attr.ib()
+
+        @attr.s
+        class B(A):
+            pass
+
+        getattr(A(42), op)(A(42))
+        getattr(B(42), op)(B(42))
+
+        assert [] == recwarn.list
+
+        getattr(A(42), op)(B(42))
+
+        w = recwarn.pop()
+
+        assert [] == recwarn.list
+        assert isinstance(w.message, DeprecationWarning)
+        assert (
+            "Comparision of subclasses using %s is deprecated and will be "
+            "removed in 2019." % (op,)
+        ) == w.message.args[0]
