@@ -9,8 +9,8 @@ import zope.interface
 
 import attr
 
-from attr import validators as validator_module
 from attr import has
+from attr import validators as validator_module
 from attr._compat import TYPE
 from attr.validators import and_, in_, instance_of, optional, provides
 
@@ -21,6 +21,7 @@ class TestInstanceOf(object):
     """
     Tests for `instance_of`.
     """
+
     def test_success(self):
         """
         Nothing happens if types match.
@@ -47,8 +48,9 @@ class TestInstanceOf(object):
         assert (
             "'test' must be <{type} 'int'> (got '42' that is a <{type} "
             "'str'>).".format(type=TYPE),
-            a, int, "42",
-
+            a,
+            int,
+            "42",
         ) == e.value.args
 
     def test_repr(self):
@@ -57,8 +59,7 @@ class TestInstanceOf(object):
         """
         v = instance_of(int)
         assert (
-            "<instance_of validator for type <{type} 'int'>>"
-            .format(type=TYPE)
+            "<instance_of validator for type <{type} 'int'>>".format(type=TYPE)
         ) == repr(v)
 
 
@@ -72,7 +73,7 @@ def always_fail(_, __, ___):
     """
     Toy validator that always fails.
     """
-    0/0
+    0 / 0
 
 
 class TestAnd(object):
@@ -97,14 +98,11 @@ class TestAnd(object):
         """
         `and_(v1, v2, v3)` and `[v1, v2, v3]` are equivalent.
         """
+
         @attr.s
         class C(object):
-            a1 = attr.ib("a1", validator=and_(
-                instance_of(int),
-            ))
-            a2 = attr.ib("a2", validator=[
-                instance_of(int),
-            ])
+            a1 = attr.ib("a1", validator=and_(instance_of(int)))
+            a2 = attr.ib("a2", validator=[instance_of(int)])
 
         assert C.__attrs_attrs__[0].validator == C.__attrs_attrs__[1].validator
 
@@ -113,6 +111,7 @@ class IFoo(zope.interface.Interface):
     """
     An interface.
     """
+
     def f():
         """
         A function called f.
@@ -123,10 +122,12 @@ class TestProvides(object):
     """
     Tests for `provides`.
     """
+
     def test_success(self):
         """
         Nothing happens if value provides requested interface.
         """
+
         @zope.interface.implementer(IFoo)
         class C(object):
             def f(self):
@@ -146,9 +147,12 @@ class TestProvides(object):
         with pytest.raises(TypeError) as e:
             v(None, a, value)
         assert (
-            "'x' must provide {interface!r} which {value!r} doesn't."
-            .format(interface=IFoo, value=value),
-            a, IFoo, value,
+            "'x' must provide {interface!r} which {value!r} doesn't.".format(
+                interface=IFoo, value=value
+            ),
+            a,
+            IFoo,
+            value,
         ) == e.value.args
 
     def test_repr(self):
@@ -157,19 +161,20 @@ class TestProvides(object):
         """
         v = provides(IFoo)
         assert (
-            "<provides validator for interface {interface!r}>"
-            .format(interface=IFoo)
+            "<provides validator for interface {interface!r}>".format(
+                interface=IFoo
+            )
         ) == repr(v)
 
 
-@pytest.mark.parametrize("validator", [
-    instance_of(int),
-    [always_pass, instance_of(int)],
-])
+@pytest.mark.parametrize(
+    "validator", [instance_of(int), [always_pass, instance_of(int)]]
+)
 class TestOptional(object):
     """
     Tests for `optional`.
     """
+
     def test_success(self, validator):
         """
         Nothing happens if validator succeeds.
@@ -195,8 +200,9 @@ class TestOptional(object):
         assert (
             "'test' must be <{type} 'int'> (got '42' that is a <{type} "
             "'str'>).".format(type=TYPE),
-            a, int, "42",
-
+            a,
+            int,
+            "42",
         ) == e.value.args
 
     def test_repr(self, validator):
@@ -206,23 +212,24 @@ class TestOptional(object):
         v = optional(validator)
 
         if isinstance(validator, list):
-            assert (
-                ("<optional validator for _AndValidator(_validators=[{func}, "
-                 "<instance_of validator for type <{type} 'int'>>]) or None>")
-                .format(func=repr(always_pass), type=TYPE)
-            ) == repr(v)
+            repr_s = (
+                "<optional validator for _AndValidator(_validators=[{func}, "
+                "<instance_of validator for type <{type} 'int'>>]) or None>"
+            ).format(func=repr(always_pass), type=TYPE)
         else:
-            assert (
-                ("<optional validator for <instance_of validator for type "
-                 "<{type} 'int'>> or None>")
-                .format(type=TYPE)
-            ) == repr(v)
+            repr_s = (
+                "<optional validator for <instance_of validator for type "
+                "<{type} 'int'>> or None>"
+            ).format(type=TYPE)
+
+        assert repr_s == repr(v)
 
 
 class TestIn_(object):
     """
     Tests for `in_`.
     """
+
     def test_success_with_value(self):
         """
         If the value is in our options, nothing happens.
@@ -239,18 +246,25 @@ class TestIn_(object):
         a = simple_attr("test")
         with pytest.raises(ValueError) as e:
             v(None, a, None)
-        assert (
-            "'test' must be in [1, 2, 3] (got None)",
-        ) == e.value.args
+        assert ("'test' must be in [1, 2, 3] (got None)",) == e.value.args
+
+    def test_fail_with_string(self):
+        """
+        Raise ValueError if the value is outside our options when the
+        options are specified as a string and the value is not a string.
+        """
+        v = in_("abc")
+        a = simple_attr("test")
+        with pytest.raises(ValueError) as e:
+            v(None, a, None)
+        assert ("'test' must be in 'abc' (got None)",) == e.value.args
 
     def test_repr(self):
         """
         Returned validator has a useful `__repr__`.
         """
         v = in_([3, 4, 5])
-        assert(
-            ("<in_ validator with options [3, 4, 5]>")
-        ) == repr(v)
+        assert (("<in_ validator with options [3, 4, 5]>")) == repr(v)
 
 
 def test_hashability():
@@ -261,6 +275,6 @@ def test_hashability():
         obj = getattr(validator_module, obj_name)
         if not has(obj):
             continue
-        hash_func = getattr(obj, '__hash__', None)
+        hash_func = getattr(obj, "__hash__", None)
         assert hash_func is not None
         assert hash_func is not object.__hash__
